/* MAME compatibility driver.h for namco.c */

#ifndef DRIVER_H
#define DRIVER_H

#include "pacifi3d.h"

/* define mame types as SDL ones */
#define offs_t    int
#define data8_t   Uint8
#define UINT32    Uint32
#define INT16     Sint16


/* mame read/write handler defines */
#define READ8_HANDLER(name) 	data8_t  name(offs_t offset)
#define WRITE8_HANDLER(name) 	void     name(offs_t offset, data8_t data)

/* define sound_region */
#define NAMCO 1
#define REGION_SOUND1 1
#define memory_region(region) 	soundprom



/* mixer pan constants */
#define MIXER_PAN_CENTER  0
#define MIXER_PAN_LEFT    1
#define MIXER_PAN_RIGHT   2
#define MIXER(level,pan) ((level & 0xff) | ((pan & 0x03) << 8))
#define MIXER_MAX_CHANNELS 1


/* use real malloc instead of auto_malloc */
#define auto_malloc malloc


/* define logerror as printf */
#if 0
    #define logerror(...) printf(__VA_ARGS__)
#else
    #define logerror(...)
#endif

/* machinesound struct */
struct MachineSound
{
	int sound_type;
	void *sound_interface;
	const char *tag;
};

#define sound_name(MachineSound) ((MachineSound)->tag)


/* streams functions */
int streams_sh_start(void);
void streams_sh_stop(void);
void streams_sh_update(void);
int stream_init(const char *name,int default_mixing_level,
		int sample_rate,
		int param,void (*callback)(int param,INT16 *buffer,int length));
int stream_init_multi(int channels,const char **names,const int *default_mixing_levels,
		int sample_rate,
		int param,void (*callback)(int param,INT16 **buffer,int length));
void stream_update(int channel,int min_interval);


/* namco.c only includes driver.h so include .h file here */
#include "namco.h"


#endif /* DRIVER_H */
