/*
 * ghost.c
 *
 * OpenGL 3D model of pacman ghost
 *
 * The ghost is 2.1 * RADIUS high
 *              2 * RADIUS wide
 */

#include "SDL_opengl.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SDL.h"

#define SPRITE
#include "video.h"
#include "video3d.h"

#define RADIUS (8.0)

#define LIST(x) (displaylists + (x))

enum {X, Y, Z} axes;
enum {HEAD, SKIRT, EYES, MAXLIST} lists;

/* GLU objects */
static GLUquadricObj *eye;
static GLuint displaylists;

static float head[SLICES][STACKS][3];
static float headnormals[SLICES][STACKS][3];
static float skirt[SLICES][3];

static float rotate[64];

extern Uint32 paldata[0x10];


int construct_ghost (void) {
    int i,j;
    double alpha, beta;
    
    displaylists = glGenLists(MAXLIST);
    eye = gluNewQuadric();
    
    /* head */
    /* calulate points for hemisphere head */
    for (i=0;i<SLICES;i++) {
        for(j=0;j<STACKS;j++) {
	    alpha = j * M_PI / STACKS / 2.0;
	    beta  = i * 2.0 * M_PI / SLICES;
	    
	    head[i][j][X] = RADIUS * cos(alpha) * cos(beta);
	    head[i][j][Y] = RADIUS * cos(alpha) * sin(beta);
	    head[i][j][Z] = RADIUS * sin(alpha);
	    
	    headnormals[i][j][X] = cos(alpha) * cos(beta);
	    headnormals[i][j][Y] = cos(alpha) * sin(beta);
	    headnormals[i][j][Z] = sin(alpha);
        }
    }
    
    /* draw head here */
    glNewList(LIST(HEAD), GL_COMPILE);
    
    /* main bit of hemisphere */
    for (j=0; j<STACKS-1; j++) {
        glBegin(GL_QUAD_STRIP);
        for (i=0; i<SLICES-1; i++) {
            glNormal3fv(headnormals[i][j]);
            glVertex3fv(head[i][j]);
            glNormal3fv(headnormals[i][j+1]);
            glVertex3fv(head[i][j+1]);
        }
        glNormal3fv(headnormals[0][j]);
        glVertex3fv(head[0][j]);
        glNormal3fv(headnormals[0][j+1]);
        glVertex3fv(head[0][j+1]);
        glEnd();
    }

    /* top of hemisphere */
    glBegin(GL_TRIANGLE_FAN);
    glNormal3d(0,1.0,0);
    glVertex3f(0,RADIUS,0);
    for(i=SLICES-1; i>=0; i--) {
        glNormal3fv(headnormals[i][STACKS-1]);
        glVertex3fv(head[i][STACKS-1]);
    }
    glNormal3fv(headnormals[SLICES-1][STACKS-1]);
    glVertex3fv(head[SLICES-1][STACKS-1]);
    glEnd();
    glEndList();

    /* skirt */
    /* calculate points for skirt */
    /* there are eight full waves on the skirt */
    for (i=0;i<SLICES;i++) {
        alpha = (i * 8 * 2 * M_PI / SLICES) - (M_PI / 2.0);

        skirt[i][X] = head[i][0][X];
	skirt[i][Y] = head[i][0][Y];
        skirt[i][Z] = head[i][0][Z] - (1.0 * RADIUS) + (0.1 * RADIUS * sin(alpha));

    }

    /* draw skirt here */
    /* the skirt is a quad strip stretching from the list of points in the
     * skirt array to the bottom row of the head */

    glNewList(LIST(SKIRT), GL_COMPILE);
    glBegin(GL_QUAD_STRIP);
    
    for (i=0;i<SLICES;i++) {
        glNormal3fv(headnormals[i][0]);
        glVertex3fv(skirt[i]);
        glNormal3fv(headnormals[i][0]);
        glVertex3fv(head[i][0]);
    }
    glNormal3fv(headnormals[0][0]);
    glVertex3fv(skirt[0]);
    glNormal3fv(headnormals[0][0]);
    glVertex3fv(head[0][0]);
    
    glEnd();
    glEndList();
    
    glNewList(LIST(EYES), GL_COMPILE);
    glPushMatrix();
    glTranslatef(head[SLICES/15][STACKS/5][X],head[SLICES/15][STACKS/5][Y],head[SLICES/15][STACKS/5][Z]-(RADIUS/8.0));
    gluSphere(eye,2.0,SLICES,STACKS);
    glPopMatrix();
    glPushMatrix();
    glTranslatef(head[SLICES-(SLICES/15)][STACKS/5][X],head[SLICES-(SLICES/15)][STACKS/5][Y],head[SLICES-(SLICES/15)][STACKS/5][Z]-(RADIUS/8.0));
    gluSphere(eye,2.0,SLICES,STACKS);
    glPopMatrix();
    glEndList();
    
    
    rotate[32]=rotate[33]=0.0;   /* to right */
    rotate[34]=rotate[35]=90.0;  /* down */
    rotate[36]=rotate[37]=180.0; /* left */
    rotate[38]=rotate[39]=270.0; /* up */
    
    return 1;
}


/* eyes color 1 */
/* pupils color 2 */
/* main body color 3 */

void draw_ghost (int sprite, int color) {
    glPushMatrix();
    glTranslatef(0.0,0.0,0.5*RADIUS);
    
    /* rotate sprite to point in correct direction */
    glRotatef(rotate[sprite],0.0,0.0,-1.0);
    
    /* eyes */
    COLORGL(1);
    glCallList(LIST(EYES));

    /* main body */
    if (colortable[color+3] != 0) {
      COLORGL(3);
      glCallList(LIST(HEAD));
      if (sprite & 0x01) {
	glRotatef(360.0/16.0,0.0,0.0,1.0);
      }
      glCallList(LIST(SKIRT));
    }
    glPopMatrix();
    return;
}


/* like a ghost but the main body is in color 2 */
/* also has a zigzag mouth and pointy eyes (not implemented) */
/* maybe use texture ?  */
void draw_scared_ghost (int sprite, int color) {
    glPushMatrix();
    glTranslatef(0.0,0.0,0.5*RADIUS);
    COLORGL(2);
    glCallList(LIST(HEAD));
    COLORGL(2);
    if (sprite & 0x01) {
      glRotatef(360.0/16.0,0.0,0.0,1.0);
    }
    glCallList(LIST(SKIRT));
    glPopMatrix();
    return;
}
