/**************************************
****   INPUT.C  -  Input devices   ****
**************************************/

/*-- Include Files ---------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include "SDL.h"
#include "pacifi3d.h"


/* Joystick definitions */
#define NUMJOYSTICKS    2
/* axes */
#define NUMAXES         2
#define AXISMIN         0
#define AXISMAX         65536
#define AXISCENTRE      AXISMAX / 2
#define AXISTHRESHOLD   AXISCENTRE / 10   /* 10% joystick threshold */
/* buttons */
#define NUMJOYBUTTONS   10


/*--------------------------------------------------------------------------*/
#define P1UP    0x00000001
#define P1LEFT  0x00000002
#define P1RIGHT 0x00000004
#define P1DOWN  0x00000008

#define P1MOTION (P1UP|P1LEFT|P1RIGHT|P1DOWN)

#define COIN1   0x00000020
#define COIN2   0x00000040
#define SERVICE 0x00000080

#define P2UP    0x00000100
#define P2LEFT  0x00000200
#define P2RIGHT 0x00000400
#define P2DOWN  0x00000800

#define P2MOTION (P2UP|P2LEFT|P2RIGHT|P2DOWN)

#define P1START 0x00002000
#define P2START 0x00004000

#define PLAYERMOTION (P1MOTION|P2MOTION)

#define SPECIAL 0x01000000


/*--------------------------------------------------------------------------*/
static Uint32 keys   =(Uint32)-1;
static Uint32 keyup  [SDLK_LAST];
static Uint32 keydown[SDLK_LAST];

static SDL_Joystick *joystick[NUMJOYSTICKS];
static Uint32 joydown[NUMJOYSTICKS][NUMJOYBUTTONS];
static Uint32 joyup  [NUMJOYSTICKS][NUMJOYBUTTONS];

static Uint32 joymask[NUMJOYSTICKS][NUMAXES][AXISMAX];
static Uint32 joyset [NUMJOYSTICKS][NUMAXES][AXISMAX];


static Uint32 turnkey[4][9];


static Uint32 direction=0;

            
/*--------------------------------------------------------------------------*/
void input_init(void) {
    int i;
    
    /* Player 1 */
    keyup[SDLK_UP]    =P1UP;    
    keyup[SDLK_DOWN]  =P1DOWN;
    keyup[SDLK_LEFT]  =P1LEFT;
    keyup[SDLK_RIGHT] =P1RIGHT;
    keyup[SDLK_1]     =P1START;        joyup[0][8]=P1START;
    keyup[SDLK_5]     =COIN1;          joyup[0][9]=COIN1;
    keyup[SDLK_9]     =SERVICE;

    /* Player 2 */
    keyup[SDLK_r]     =P2UP;
    keyup[SDLK_f]     =P2DOWN;
    keyup[SDLK_d]     =P2LEFT;
    keyup[SDLK_g]     =P2RIGHT;
    keyup[SDLK_2]     =P2START;        joyup[1][8]=P2START;
    keyup[SDLK_6]     =COIN2;          joyup[1][9]=COIN2;
    
    /* Special */
    keyup[SDLK_F1]    =SPECIAL;        keyup[SDLK_F7]    =SPECIAL;
    keyup[SDLK_F2]    =SPECIAL;        keyup[SDLK_F8]    =SPECIAL;
    keyup[SDLK_F3]    =SPECIAL;        keyup[SDLK_F9]    =SPECIAL;
    keyup[SDLK_F4]    =SPECIAL;        keyup[SDLK_F10]   =SPECIAL;
    keyup[SDLK_F5]    =SPECIAL;        keyup[SDLK_F11]   =SPECIAL;
    keyup[SDLK_F6]    =SPECIAL;        keyup[SDLK_F12]   =SPECIAL;

    keyup[SDLK_ESCAPE]=SPECIAL;
        
    /* set key down mask */
    for(i=0;i<SDLK_LAST;i++) {
        keydown[i]=~keyup[i];
    }
    
    /* set joy button down mask */
    for(i=0; i<NUMJOYBUTTONS; i++) {
        joydown[0][i]=~joyup[0][i];
        joydown[1][i]=~joyup[0][i];
    }
    
    /* configure joystick axes */
    /* left and up */        
    for(i=AXISMIN; i<AXISCENTRE-AXISTHRESHOLD; i++) {
        joymask[0][0][i]=~P1LEFT;
        joymask[1][0][i]=~P2LEFT;
        joymask[0][1][i]=~P1UP;
        joymask[1][1][i]=~P2UP;
        joyset[0][0][i]=P1RIGHT;
        joyset[1][0][i]=P2RIGHT;
        joyset[0][1][i]=P1DOWN;
        joyset[1][1][i]=P2DOWN;        
    }
    
    /* centre */
    for(i=AXISCENTRE-AXISTHRESHOLD; i<AXISCENTRE+AXISTHRESHOLD; i++) {
        joymask[0][0][i]=~0;
        joymask[1][0][i]=~0;
        joymask[0][1][i]=~0;
        joymask[1][1][i]=~0;
        joyset[0][0][i]=P1RIGHT|P1LEFT;
        joyset[1][0][i]=P2RIGHT|P2LEFT;
        joyset[0][1][i]=P1DOWN|P1UP;
        joyset[1][1][i]=P2DOWN|P2UP;        
    }
    
    /* right and down */
    for(i=AXISCENTRE+AXISTHRESHOLD; i<AXISMAX; i++) {
        joymask[0][0][i]=~P1RIGHT;
        joymask[1][0][i]=~P2RIGHT;
        joymask[0][1][i]=~P1DOWN;
        joymask[1][1][i]=~P2DOWN;
        joyset[0][0][i]=P1LEFT;
        joyset[1][0][i]=P2LEFT;
        joyset[0][1][i]=P1UP;
        joyset[1][1][i]=P2UP;            
    }
        
    
    /* open joysticks */
    for(i=0; i<SDL_NumJoysticks() && i<NUMJOYSTICKS; i++) {
        joystick[i]=SDL_JoystickOpen(i);
        if(joystick[i]!=NULL) {
            printf("\nOpened Joystick %d\n",i);
            printf("Name: %s\n", SDL_JoystickName(i));
            printf("Number of Axes: %d\n", SDL_JoystickNumAxes(joystick[i]));
            printf("Number of Buttons: %d\n", SDL_JoystickNumButtons(joystick[i]));
        }
    }


    /* turn keys */
    turnkey[NORTH][0]=turnkey[EAST][0]=turnkey[SOUTH][0]=turnkey[WEST][0]=0;
    /* North */
    turnkey[NORTH][P1UP]=P1UP;
    turnkey[NORTH][P1LEFT]=P1LEFT;
    turnkey[NORTH][P1RIGHT]=P1RIGHT;
    turnkey[NORTH][P1DOWN]=P1DOWN;

    /* East */
    turnkey[EAST][P1UP]=P1RIGHT;
    turnkey[EAST][P1LEFT]=P1UP;
    turnkey[EAST][P1RIGHT]=P1DOWN;
    turnkey[EAST][P1DOWN]=P1LEFT;

    /* South */
    turnkey[SOUTH][P1UP]=P1DOWN;
    turnkey[SOUTH][P1LEFT]=P1RIGHT;
    turnkey[SOUTH][P1RIGHT]=P1LEFT;
    turnkey[SOUTH][P1DOWN]=P1UP;

    /* West */
    turnkey[WEST][P1UP]=P1LEFT;
    turnkey[WEST][P1LEFT]=P1DOWN;
    turnkey[WEST][P1RIGHT]=P1UP;
    turnkey[WEST][P1DOWN]=P1RIGHT;

    
}


void input_shutdown(void) {
    int i;
    
    /* Close joysticks */
    for(i=0; i<NUMJOYSTICKS; i++) {
        if(joystick[i]!=NULL) {
            SDL_JoystickClose(joystick[i]);    
            joystick[i]=NULL;
        }
    }
}


void input_direction(int bearing) {
    direction = bearing;
}


INLINE void specialKey (SDLKey key) {
    switch(key) {
        case SDLK_F1: video_mode_toggle(); break;
        case SDLK_F11: video_fullscreen_toggle(); break;
        case SDLK_F2: pacman_reset(); break;
        case SDLK_F3: sound_toggle(); break;
        case SDLK_F5: video_inc_tile(); break;
        case SDLK_F6: video_dec_tile(); break;
        case SDLK_F12: video_save_snapshot(); break;
        case SDLK_ESCAPE: exit(0); break;
        default:
            break;
    }
}


INLINE void keyDown (SDLKey key) {
    Uint32 val=~keydown[key];
    /* process special keys */
    if(val & SPECIAL) {
        specialKey(key);
    
    /* process direction keys */
    } else if (val & P1MOTION) {
        val=turnkey[direction][val];
	keyup[key]=val;
        keys&=~val;
    } else if (val & P2MOTION) {
        val=turnkey[direction][val>>8]<<8;
	keyup[key]=val;
        keys&=~val;
    /* process other keys */
    } else {
        keys &= keydown[key];
    }
}


INLINE void keyUp (SDLKey key) {
    keys |= keyup[key];
}


INLINE void joyDown (int which, int button) {
    if (which<NUMJOYSTICKS && button<NUMJOYBUTTONS) {
        keys &= joydown[which][button];
    }
}


INLINE void joyUp (int which, int button) {
    if (which<NUMJOYSTICKS && button<NUMJOYBUTTONS) {
        keys |= joyup[which][button];
    }
}


INLINE void joyMotion (int which, int axis, int value) {
    value+=AXISCENTRE;
    if (which<NUMJOYSTICKS && axis <NUMAXES && value<AXISMAX) {
        keys &= joymask[which][axis][value];
        keys |= joyset[which][axis][value];
    }
}


static int mousegrabbed=0;
static Uint16 mousex, mousey;
INLINE void mouseDown (Uint8 button, Uint16 x, Uint16 y) {
    mousegrabbed=1;
    mousex=x;
    mousey=y;
    SDL_ShowCursor(SDL_DISABLE);
    SDL_WM_GrabInput(SDL_GRAB_ON);
}

INLINE void mouseUp (Uint8 button) {
    mousegrabbed=0;
    SDL_ShowCursor(SDL_ENABLE);
    SDL_WM_GrabInput(SDL_GRAB_OFF);
    SDL_WarpMouse(mousex,mousey);
}

INLINE void mouseMotion (Sint16 xrel, Sint16 yrel) {
    if (mousegrabbed == 1) {
        video_mouse_rot(xrel,yrel);
    }
}

void processEvents(void) {
    SDL_Event event;
        
    while(SDL_PollEvent(&event)) {
        switch(event.type) {
            case SDL_KEYDOWN: keyDown(event.key.keysym.sym); break;
            case SDL_KEYUP:   keyUp(event.key.keysym.sym); break;
            
            case SDL_JOYBUTTONDOWN: joyDown(event.jbutton.which, event.jbutton.button); break;
            case SDL_JOYBUTTONUP:   joyUp(event.jbutton.which, event.jbutton.button); break;

            case SDL_JOYAXISMOTION: joyMotion(event.jaxis.which, event.jaxis.axis, event.jaxis.value); break;
			
	    case SDL_MOUSEBUTTONDOWN: mouseDown(event.button.button,event.button.x,event.button.y); break;
	    case SDL_MOUSEBUTTONUP: mouseUp(event.button.button); break;
	    case SDL_MOUSEMOTION: mouseMotion(event.motion.xrel, event.motion.yrel); break;
            
            case SDL_VIDEORESIZE: setup_opengl(event.resize.w,event.resize.h); break;

            case SDL_QUIT:    exit(0); break;
            
            default:
                break;
        }
    }
}

/*--------------------------------------------------------------------------*/
Uint8 input_port_0_r (int Addr) {
    return keys&0xff;
}

/*--------------------------------------------------------------------------*/
Uint8 input_port_1_r (int Addr) {
    return (keys>>8)&0xff;
}

/*--------------------------------------------------------------------------*/
Uint8 input_port_2_r (int Addr) {
    return (keys>>16)&0xff;
}

/*--------------------------------------------------------------------------*/
Uint8 input_port_3_r (int Addr) {
    return 0xff;
}



