/************************
*** Z80 CPU Interface ***
************************/

//-- Include Files ----------------------------------------------------------
#include 	<stdio.h>
#include 	<stdlib.h>
#include	"pacifi3d.h"
#include        "z80/Z80.h"

/* includes for sound */
#include        "driver.h"
#include        "namco.h"

//-- Variables --------------------------------------------------------------
Uint8		memory[0x10000];
Uint8           soundprom[0x200];
int		videoram_size=0x400;
int		spriteram_size=0x10;
Uint8           dirtybuffer[0x400];
Uint8         * videoram=&memory[0x4000];
Uint8         * colorram=&memory[0x4400];
Uint8	      * spriteram=&memory[0x4ff0];
Uint8         * spriteram_2=&memory[0x5060];

//-- Exported Functions -----------------------------------------------------

/** RdZ80()/WrZ80() ******************************************/
/** These functions are called when access to RAM occurs.   **/
/** They allow to control memory access.                    **/
/************************************ TO BE WRITTEN BY USER **/
void WrZ80(register word Addr,register byte Value);
byte RdZ80(register word Addr);

/** InZ80()/OutZ80() *****************************************/
/** Z80 emulation calls these functions to read/write from  **/
/** I/O ports. There can be 65536 I/O ports, but only first **/
/** 256 are usually used                                    **/
/************************************ TO BE WRITTEN BY USER **/
void OutZ80(register word Port,register byte Value);
byte InZ80(register word Port);


typedef Uint8 (*readhandler)(int Addr);
typedef void (*writehandler)(int Addr,Uint8 Value);

readhandler readmem[0x10000];
writehandler writemem[0x10000];


/* read functions */
Uint8 memory_r (int Addr);
/* write functions */
void memory_w (int Addr,Uint8 Value);
void MWA_NOP (int Addr,Uint8 Value);
void videoram_w (int Addr,Uint8 Value);
void colorram_w (int Addr,Uint8 Value);
void sound_enable_w (int Addr,Uint8 Value);
void sound_w (int Addr,Uint8 Value);


//---------------------------------------------------------------------------
void memory_init(void)
{
    int i;
    /* init memory space */
    for(i=0;i<0x10000;i++) {
        memory[i]=0;
        readmem[i]=memory_r;
        writemem[i]=memory_w;
    }
    
    /* inputs */
    for(i=0x0000;i<=0x003f;i++) {
        readmem[i+0x5000]=input_port_0_r;
        readmem[i+0x5040]=input_port_1_r;
        readmem[i+0x5080]=input_port_2_r;
        readmem[i+0x50c0]=input_port_3_r;
    }
    
    /* video hardware */
    for(i=0x0000;i<=0x03ff;i++) {
        writemem[i+0x4000]=videoram_w;
        writemem[i+0x4400]=colorram_w;
        writemem[i+0xc000]=videoram_w;
        writemem[i+0xc400]=colorram_w;
    }
    
    /* sound hardware */
    writemem[0x5001]=sound_enable_w;
    pengo_soundregs=&memory[0x5040];
    for(i=0x5040; i<=0x505f; i++) {
        writemem[i]=sound_w;
    }
    
    /* other */
    writemem[0x5002]=MWA_NOP;
    writemem[0xffff]=MWA_NOP;
}



/* Memory read Functions */
Uint8 memory_r(int Addr)
{
    return memory[Addr];
}


/* Memory Write Functions */
void MWA_NOP (int Addr, Uint8 Value) {return;}

void memory_w (int Addr, Uint8 Value) {
    memory[(int)Addr]=Value;
}

void videoram_w (int Addr, Uint8 Value) {
    Addr&=0x3ff;
    if (videoram[Addr]!=Value) {
	dirtybuffer[Addr]=1;
        videoram[Addr]=Value;
    }
}

void colorram_w (int Addr, Uint8 Value) {
    Addr=(Addr - 0x400)&0x3ff;
    if (colorram[Addr]!=Value) {
	dirtybuffer[Addr]=1;
        colorram[Addr]=Value;
    }
}

void sound_enable_w (int Addr,Uint8 Value) {
    pengo_sound_enable_w(Addr,Value);
}

void sound_w (int Addr,Uint8 Value) {
    Addr = (Addr - 0x5040) & 0x1f;
    pengo_sound_w(Addr, Value);
}


/* Memory */
byte RdZ80(register word Addr) {
     return (readmem[(int)Addr])((int)Addr);
}

void WrZ80(register word Addr,register byte Value) {
     (writemem[(int)Addr])((int)Addr,(Uint8)Value);
}



/* Port Functions */
void OutZ80(register word Port,register byte Value) {
    if (Port == 0) {
        set_interrupt_vector(Value);
    } else {
        fprintf(stderr,"Unhandled port write address=0x%04x data=0x%02x\n",Port,Value);
    }
}


byte InZ80(register word Port) {
     fprintf(stderr,"Unhandled port read address=0x%04x\n",Port);
     return (byte)-1;
}


void PatchZ80(register Z80 *R) {
    ;
}
