/****************************************
****  pacifi3d.c  - Main Source File  ***
*****************************************/

//-- Include Files -----------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "SDL.h"
#include <ctype.h>
#include <string.h>

#include "pacifi3d.h"
#include "z80/Z80.h"
#include "icon.h"
#include "rom.h"



//-- Private variables -------------------------------------------------------
static Z80 z80;
static Uint32 old_time;

static int interrupt_vector=INT_IRQ;

//-- Function Prototypes -----------------------------------------------------
void pacman_reset(void);
void pacman_run(void);
void pacman_shutdown(void);

//----------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    SDL_RWops *iconfp;
    SDL_Surface *icon_surface;

    /* Initialise SDL */
    if((SDL_Init(SDL_INIT_JOYSTICK)==-1)) {
        printf("Could not initialize SDL: %s.\n", SDL_GetError());
        exit(-1);
    }

    /* Displays version number, date and time */
    puts(VERSION1);
    puts(VERSION2);

    /* Register exit procedure */
    atexit(pacman_shutdown);

    /* memory init */
    memory_init();

    /* Load ROMS */
    if (argc > 1) {
        load_game(argv[1]);
    } else {
        load_game("puckman");
    }

    /* Video init */
    video_init();

    /* set up window icon */
    iconfp=SDL_RWFromMem((void*)icon, sizeof(icon));
    icon_surface=SDL_LoadBMP_RW(iconfp,0);
    SDL_SetColorKey(icon_surface, SDL_SRCCOLORKEY, SDL_MapRGB(icon_surface->format,0,0,0));

    SDL_WM_SetIcon(icon_surface,NULL);

    /* set window title */
    SDL_WM_SetCaption(VERSION1,VERSION1);

    /* Initialise input */
    input_init();

    /* reset hardware */
    pacman_reset();
    
    /* init sound */
    sound_init();

    /* get time for speed throttle */
    old_time=SDL_GetTicks();

    /* Enter Z80 loop */
    z80.IAutoReset=1;
    z80.IPeriod=(int)(Z80_VBL_CYCLES);
    RunZ80(&z80);

    return 0;
}


//----------------------------------------------------------------------------
void pacman_reset(void)
{
    ResetZ80(&z80);
}

//----------------------------------------------------------------------------
void pacman_shutdown(void)
{
    // Close everything and free memory
    input_shutdown();
    video_shutdown();
    sound_stop();

    printf("pacman: System Shutdown.\n");

    SDL_Quit();
}


void set_interrupt_vector(int vector) 
{
    interrupt_vector=vector;
}


float amount_frame_complete(void) {
   return ((Z80_VBL_CYCLES)-z80.ICount)/(Z80_VBL_CYCLES);
}


//----------------------------------------------------------------------------
/* z80 loop */
/* This procedure is called every Z80_VBL_CYCLES by the Z80 emulation */
word LoopZ80(register Z80 *R)
{


    Sint32 remaining;

    /* Call display routine */
    video_draw_screen();

    /* Update Sound */
    sound_update();

    /* Update keys and Joystick */
    processEvents();

    /* Speed Throttle */
    remaining = (old_time + REFRESHTIME) - SDL_GetTicks();
    if (remaining > 0) {
      SDL_Delay(remaining);
    }
    old_time = SDL_GetTicks();
    
    /* raise IRQ */
    return interrupt_vector;
}


