/*
 * pac.c
 *
 * OpenGL 3D model of pacman
 *
 * Pacman is 2 * RADIUS wide
 */


#include "SDL_opengl.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "SDL.h"

#define SPRITE
#include "video.h"
#include "video3d.h"

#define RADIUS (8.0)

enum {X, Y, Z} axes;

static GLuint pacmanlists[64];
static GLuint mouthlists[64];

static float mesh[SLICES][STACKS][3];
static float normals[SLICES][STACKS][3];

static void generate_pacman(int sprite, float mouth_angle, float rotz, float roty) {
    int i, j;
    int offset = mouth_angle / 360.0 * SLICES / 2.0;

    pacmanlists[sprite] = glGenLists(1);

    glNewList(pacmanlists[sprite], GL_COMPILE);
    glPushMatrix();
    glRotatef(rotz,0.0,0.0,1.0);
    glRotatef(roty,0.0,1.0,0.0);


    /* main bit of sphere */
    for (j=0; j<STACKS-1; j++) {
	glBegin(GL_QUAD_STRIP);
	for (i=offset; i<SLICES-offset; i++) {
            glNormal3fv(normals[i][j+1]);
            glVertex3fv(mesh[i][j+1]);
            glNormal3fv(normals[i][j]);
	    glVertex3fv(mesh[i][j]);

	}
	glNormal3fv(normals[i%SLICES][j+1]);
	glVertex3fv(mesh[i%SLICES][j+1]);
	glNormal3fv(normals[i%SLICES][j]);
	glVertex3fv(mesh[i%SLICES][j]);

	glEnd();
    }

    /* top of sphere */
    glBegin(GL_TRIANGLE_FAN);
    glNormal3f(0,1.0,0);
    glVertex3f(0,RADIUS,0);
    for (i=offset; i<SLICES-offset; i++) {
        glNormal3fv(normals[i][STACKS-1]);
	glVertex3fv(mesh[i][STACKS-1]);
    }
    glNormal3fv(normals[i%SLICES][STACKS-1]);
    glVertex3fv(mesh[i%SLICES][STACKS-1]);
    glEnd();

    glPopMatrix();
    glEndList();


    mouthlists[sprite] = glGenLists(1);

    glNewList(mouthlists[sprite], GL_COMPILE);

    glPushMatrix();
    glRotatef(rotz,0.0,0.0,1.0);
    glRotatef(roty,0.0,1.0,0.0);

    /* inside mouth */
    if (offset > 0 && offset < SLICES) {
        glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0,0,0);
	glVertex3f(0,RADIUS,0);
	for(j=0;j<STACKS;j++) {
	  glVertex3fv(mesh[offset][j]);
	}
	glVertex3f(0,RADIUS,0);
	for(j=STACKS-1;j>=0;j--) {
	  glVertex3fv(mesh[SLICES-offset][j]);
	}
	glVertex3d(0,-RADIUS,0);
	glEnd();
    }
    glPopMatrix();
    glEndList();



}


int construct_pacman (void) {
    int i, j;
    double alpha, beta;

    /* calulate points for sphere */
    for (i=0;i<SLICES;i++) {
        for(j=0;j<STACKS;j++) {
	    alpha = (j * M_PI / STACKS) - (M_PI / 2.0);
	    beta  = i * 2.0 * M_PI / SLICES;

            mesh[i][j][X] = RADIUS * cos(alpha) * cos(beta);
	    mesh[i][j][Y] = RADIUS * sin(alpha);
	    mesh[i][j][Z] = RADIUS * cos(alpha) * sin(beta);
	    
	    normals[i][j][X] = cos(alpha) * cos(beta);
	    normals[i][j][Y] = sin(alpha);
	    normals[i][j][Z] = cos(alpha) * sin(beta);
        }
    }
    
    /* regular pacman */
    generate_pacman(44, 120.0,   0.0, 0.0); /* wide left */
    generate_pacman(45, 120.0, -90.0, 0.0); /* wide down */
    generate_pacman(46,  60.0,   0.0, 0.0); /* narrow left */
    generate_pacman(47,  60.0, -90.0, 0.0); /* narrow down */
    generate_pacman(48,   0.0,   0.0, 0.0); /* whole */
    
    /* dieing animation */
    generate_pacman(52, (360.0/11.0)*1.0,  0.0, -90.0);
    generate_pacman(53, (360.0/11.0)*2.0,  0.0, -90.0);
    generate_pacman(54, (360.0/11.0)*3.0,  0.0, -90.0);
    generate_pacman(55, (360.0/11.0)*4.0,  0.0, -90.0);
    generate_pacman(56, (360.0/11.0)*5.0,  0.0, -90.0);
    generate_pacman(57, (360.0/11.0)*6.0,  0.0, -90.0);
    generate_pacman(58, (360.0/11.0)*7.0,  0.0, -90.0);
    generate_pacman(59, (360.0/11.0)*8.0,  0.0, -90.0);
    generate_pacman(60, (360.0/11.0)*9.0,  0.0, -90.0);
    generate_pacman(61, (360.0/11.0)*10.0, 0.0, -90.0);
    generate_pacman(62, (360.0/11.0)*11.0, 0.0, -90.0);
    
    return 1;
}


void draw_pacman (int sprite, int color) {
    glPushMatrix();

    COLORGL(3);
    glCallList(pacmanlists[sprite]);

    COLORGL(0);
    /* temporarily color red */
    glColor3f(1.0,0.0,0.0);
    glCallList(mouthlists[sprite]);

    glPopMatrix();
}

void draw_big_pacman (int sprite, int color) {
    glPushMatrix();
    glScalef(2.0,2.0,2.0);
    draw_pacman(sprite, color);
    glPopMatrix();
}


