/*
 * should be able to load and play most versions of pacman.
 * however pacifi3d only works well if the game sticks to the original 
 * tiles / sprites.
 * so we have:
 *    puckman
 *    puckmana
 *    puckmanf
 *    pacman
 *    pacmanf
 *    puckmod
 *    pacmod
 *    hangly
 *    hangly2
 *    hangly3
 *    newpuckx
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pacifi3d.h"

static char gamename[10];
const char * parent="puckman";


void	rom_load(const char *name, int offset, int size, char* region, Uint32 crc32) {
    FILE *rom;
    char filename[255];
    
    sprintf(filename,"roms/%s/%s",gamename,name);
    rom=fopen(filename,"rb");

    if (rom == NULL) {
        /* try parent instead */
        sprintf(filename,"roms/%s/%s",parent,name);
        rom=fopen(filename,"rb");
        
        if (rom == NULL) {
            /* try roms directory */
            sprintf(filename,"roms/%s",name);
            rom=fopen(filename,"rb");
            
            if (rom == NULL) {
                fprintf(stderr,"Error: unable to load %s\n",name);
                exit(1);
            }
        }
    }

    fread(&region[offset],sizeof(char),size,rom);
    printf("loading %s...\n",name);
    fclose(rom);
}



void load_game(const char *name) {
    strncpy(gamename,name,sizeof(gamename));

    printf("Loading %s\n",name);

    if (!strcmp(name,"puckman")) {
      printf("Loading Puckman (Japan set 1)\n");
      rom_load("namcopac.6e", 0x0000, 0x1000, memory,    0xfee263b3);
      rom_load("namcopac.6f", 0x1000, 0x1000, memory,    0x39d1fc83);
      rom_load("namcopac.6h", 0x2000, 0x1000, memory,    0x02083b03);
      rom_load("namcopac.6j", 0x3000, 0x1000, memory,    0x7a36fe55);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);
  
      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      
      return;
    }

    if(!strcmp(name,"puckmanf")) {
      printf("Loading Puckman (Japan set 1 with speedup hack)\n");
      rom_load("namcopac.6e", 0x0000, 0x1000, memory,    0xfee263b3);
      rom_load("nampfast.6f", 0x1000, 0x1000, memory,    0x51b38db9);
      rom_load("namcopac.6h", 0x2000, 0x1000, memory,    0x02083b03);
      rom_load("namcopac.6j", 0x3000, 0x1000, memory,    0x7a36fe55);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);
  
      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }

    if(!strcmp(name,"puckmod")) {
      printf("Loading PuckMan (harder?)\n");
      rom_load("namcopac.6e", 0x0000, 0x1000, memory,    0xfee263b3);
      rom_load("namcopac.6f", 0x1000, 0x1000, memory,    0x39d1fc83);
      rom_load("namcopac.6h", 0x2000, 0x1000, memory,    0x02083b03);
      rom_load("npacmod.6j",  0x3000, 0x1000, memory,    0x7d98d5f5);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);

      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }
    
    if(!strcmp(name,"puckmana")) {
      printf("Loading PuckMan (Japan set 2)\n");
      rom_load("pacman.6e",   0x0000, 0x1000, memory,    0xc1e6ab10);
      rom_load("pacman.6f",   0x1000, 0x1000, memory,    0x1a6fb2d4);
      rom_load("pacman.6h",   0x2000, 0x1000, memory,    0xbcdd1beb);
      rom_load("prg7",        0x3000, 0x0800, memory,    0xb6289b26);
      rom_load("prg8",        0x3800, 0x0800, memory,    0x17a88c13);

      rom_load("chg1",        0x0000, 0x0800, tilerom,   0x2066a0b7);
      rom_load("chg2",        0x0800, 0x0800, tilerom,   0x3591b89d);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);

      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }

    if(!strcmp(name,"pacman")) {
      printf("Loading Pac-Man (Midway)\n");
      rom_load("pacman.6e",   0x0000, 0x1000, memory,    0xc1e6ab10);
      rom_load("pacman.6f",   0x1000, 0x1000, memory,    0x1a6fb2d4);
      rom_load("pacman.6h",   0x2000, 0x1000, memory,    0xbcdd1beb);
      rom_load("pacman.6j",   0x3000, 0x1000, memory,    0x817d94e3);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);
  
      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }
    
    if(!strcmp(name,"pacmanf")) {
      printf("Loading Pac-Man (Midway, with speedup hack)\n");
      rom_load("pacman.6e",   0x0000, 0x1000, memory,    0xc1e6ab10);
      rom_load("pacfast.6f",  0x1000, 0x1000, memory,    0x720dc3ee);
      rom_load("pacman.6h",   0x2000, 0x1000, memory,    0xbcdd1beb);
      rom_load("pacman.6j",   0x3000, 0x1000, memory,    0x817d94e3);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);
  
      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);

      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }
    
    if(!strcmp(name,"pacmod")) {
      printf("Loading Pac-Man (Midway, harder)\n");
      rom_load("pacmanh.6e",  0x0000, 0x1000, memory,    0x3b2ec270);
      rom_load("pacman.6f",   0x1000, 0x1000, memory,    0x1a6fb2d4);
      rom_load("pacmanh.6h",  0x2000, 0x1000, memory,    0x18811780);
      rom_load("pacmanh.6j",  0x3000, 0x1000, memory,    0x5c96a733);

      rom_load("pacmanh.5e",  0x0000, 0x1000, tilerom,   0x299fb17a);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);
  
      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }

    if(!strcmp(name,"hangly")) {
      printf("Loading Hangly-Man (set 1)\n");
      rom_load("hangly.6e",   0x0000, 0x1000, memory,    0x5fe8610a);
      rom_load("hangly.6f",   0x1000, 0x1000, memory,    0x73726586);
      rom_load("hangly.6h",   0x2000, 0x1000, memory,    0x4e7ef99f);
      rom_load("hangly.6j",   0x3000, 0x1000, memory,    0x7f4147e6);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);

      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }

    if(!strcmp(name,"hangly2")) {
      printf("Loading Hangly-Man (set 2)\n");
      rom_load("hangly.6e",   0x0000, 0x1000, memory,    0x5fe8610a);
      rom_load("hangly2.6f",  0x1000, 0x0800, memory,    0x5ba228bb);
      rom_load("hangly2.6m",  0x1800, 0x0800, memory,    0xbaf5461e);
      rom_load("hangly.6h",   0x2000, 0x1000, memory,    0x4e7ef99f);
      rom_load("hangly2.6j",  0x3000, 0x0800, memory,    0x51305374);
      rom_load("hangly2.6p",  0x3800, 0x0800, memory,    0x427c9d4d);

      rom_load("pacmanh.5e",  0x0000, 0x1000, tilerom,   0x299fb17a);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);

      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }
    
    if(!strcmp(name,"hangly3")) {
      printf("Loading Hangly-Man (set 3)\n");
      rom_load("hm1.6e",      0x0000, 0x0800, memory,    0x9d027c4a);
      rom_load("hm5.6k",      0x0800, 0x0800, memory,    0x194c7189);
      rom_load("hangly2.6f",  0x1000, 0x0800, memory,    0x5ba228bb);
      rom_load("hangly2.6m",  0x1800, 0x0800, memory,    0xbaf5461e);
      rom_load("hm3.6h",      0x2000, 0x0800, memory,    0x08419c4a);
      rom_load("hm7.6n",      0x2800, 0x0800, memory,    0xab74b51f);
      rom_load("hm4.6j",      0x3000, 0x0800, memory,    0x5039b082);
      rom_load("hm8.6p",      0x3800, 0x0800, memory,    0x931770d7);

      rom_load("hm9.5e",      0x0000, 0x0800, tilerom,   0x5f4be3cc);
      rom_load("hm11.5h",     0x0800, 0x0800, tilerom,   0x3591b89d);
      rom_load("hm10.5f",     0x0000, 0x0800, spriterom, 0x9e39323a);
      rom_load("hm12.5j",     0x0800, 0x0800, spriterom, 0x1b1d9096);

      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);

      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }

    if(!strcmp(name,"newpuckx")) {
      printf("Loading New Puck-X\n");
      rom_load("puckman.6e",  0x0000, 0x1000, memory,    0xa8ae23c5);
      rom_load("pacman.6f",   0x1000, 0x1000, memory,    0x1a6fb2d4);
      rom_load("puckman.6h",  0x2000, 0x1000, memory,    0x197443f8);
      rom_load("puckman.6j",  0x3000, 0x1000, memory,    0x2e64a3ba);

      rom_load("pacman.5e",   0x0000, 0x1000, tilerom,   0x0c944964);
      rom_load("pacman.5f",   0x0000, 0x1000, spriterom, 0x958fedf9);

      rom_load("82s123.7f",   0x0000, 0x0020, prom,      0x2fc650bd);
      rom_load("82s126.4a",   0x0020, 0x0100, prom,      0x3eb3a8e4);
  
      rom_load("82s126.1m",   0x0000, 0x0100, soundprom, 0xa9cc86bf);
      rom_load("82s126.3m",   0x0100, 0x0100, soundprom, 0x77245b66);
      return;
    }


    printf("Error: unknown romset: %s\n",name);
    printf("Supported sets are:\n");
    printf("  puckman\n");
    printf("  puckmana\n");
    printf("  puckmanf\n");
    printf("  pacman\n");
    printf("  pacmanf\n");
    printf("  puckmod\n");
    printf("  pacmod\n");
    printf("  hangly\n");
    printf("  hangly2\n");
    printf("  hangly3\n");
    printf("  newpuckx\n");
    exit(1);
}

