

#include "driver.h"
#include "streams.h"

#define BUFFERSIZE 16384
#define FREQUENCY 48000
#define SAMPLES 1024
static SDL_AudioSpec desired;
static Uint8 play_buffer[BUFFERSIZE];

static int snd_enabled=1;
static int snd_read  = 0;
static int snd_write = 0;


/* pacman namco interface */
static struct namco_interface namco_interface =
{
	3072000/32,	/* sample rate */
	3,		/* number of voices */
	100,		/* playback volume */
	REGION_SOUND1	/* memory region */
};


static struct MachineSound pacman_machine_sound =
{
	NAMCO,
	&namco_interface,
	"namco"
};

int snd_err=1;

/* Callback for sound */
static void _snd_callback(void* unused, Uint8 *stream, int len)
{
  int remain = BUFFERSIZE - snd_read;

  if (remain < len) {
    memcpy(stream, play_buffer+snd_read, remain);
    len -= remain;
    stream += remain;
    memcpy(stream, play_buffer, len);
    snd_read = len;
  } else {
    memcpy(stream, play_buffer+snd_read, len);
    snd_read += len;
  }
}


int sound_write(Uint8 *data, int count) {
  int remain;
  count=count*2;
  
  if (snd_enabled) {
    SDL_LockAudio();

    remain = BUFFERSIZE - snd_write;
    if (remain < count) {
       memcpy(play_buffer+snd_write,data,remain);
       count -= remain;
       data  += remain;
       memcpy(play_buffer,data,count);
       snd_write = count;
    } else {
       memcpy(play_buffer+snd_write,data,count);
       snd_write += count;
    }
    SDL_UnlockAudio();
  }
  return 0;
}


void sound_init(void) {

  if(SDL_InitSubSystem(SDL_INIT_AUDIO)) {
    fprintf(stderr,"Couldn't Start Sound.\n");
    snd_enabled=0;
  }

  memset(play_buffer, 0, sizeof(play_buffer));

  /* set desired sound mode */
  desired.freq = FREQUENCY;
  desired.samples = SAMPLES;
  desired.format = AUDIO_S16SYS;
  desired.channels = 1;
  desired.callback = _snd_callback;
  desired.userdata = NULL;

  SDL_OpenAudio(&desired, NULL);

  /* start sound hardware */
  streams_sh_start();
  namco_sh_start(&pacman_machine_sound);
  
  /* start soundcard */
  //SDL_PauseAudio(!snd_enabled);
}


void sound_toggle(void) {
  SDL_PauseAudio(snd_enabled);
  snd_enabled = !snd_enabled;
}


void sound_stop(void) {
  namco_sh_stop();
  streams_sh_stop();
}


void sound_update(void) {
  streams_sh_update();
  SDL_PauseAudio(!snd_enabled);
}

