/***************************************************************************

  streams.c

  Handle general purpose audio streams

***************************************************************************/

#include "driver.h"
#include <math.h>
#include <stdlib.h>

#define BUFFER_LEN 16384

static INT16 *stream_buffer;
static int stream_sample_rate;
static int stream_buffer_pos;
static int stream_sample_length;	/* in usec */
static int stream_param;
static int frame_length;
static void (*stream_callback)(int param,INT16 *buffer,int length);
float amount_frame_complete(void);
float samples_this_frame=0.0;



int streams_sh_start(void)
{
	stream_buffer = 0;
	return 0;
}


void streams_sh_stop(void)
{
	free(stream_buffer);
	stream_buffer = 0;
}


void streams_sh_update(void)
{

	/* update the output buffer */
	if (stream_buffer)
	{
		int len;
		
		samples_this_frame += (amount_frame_complete() * (float)frame_length);
		len = (int)samples_this_frame - stream_buffer_pos;
		//printf("samples_this_frame = %f\n",samples_this_frame);
		
		if (len > 0) {
			(*stream_callback)(stream_param,stream_buffer+stream_buffer_pos,len);
		}
		sound_write((Uint8*)stream_buffer,(int)samples_this_frame);
		
		/* adjust next frame length */
		samples_this_frame-=(int)samples_this_frame;
		/* reset stream position */
		stream_buffer_pos = 0;
	}

}


int stream_init(const char *name,int default_mixing_level,
		int sample_rate,
		int param,void (*callback)(int param,INT16 *buffer,int length))
{
	if ((stream_buffer = malloc(sizeof(INT16)*BUFFER_LEN)) == 0)
		return -1;

	stream_sample_rate = sample_rate;
	frame_length=stream_sample_rate / FPS;
	stream_buffer_pos = 0;
	if (sample_rate)
		stream_sample_length = 1000000 / sample_rate;
	else
		stream_sample_length = 0;
	stream_param = param;
	stream_callback = callback;

	return 0;
}


int stream_init_multi(int channels,const char **names,const int *default_mixing_levels,
		int sample_rate,
		int param,void (*callback)(int param,INT16 **buffer,int length))
{
	return 0;
}


/* min_interval is in usec */
void stream_update(int channel,int min_interval)
{
	/* update the output buffer */
	if (stream_buffer)
	{
		int newpos = (int)(amount_frame_complete() * (float)frame_length);
		int len = newpos - stream_buffer_pos;
		

		if (len > 0) {
		  (*stream_callback)(stream_param,stream_buffer+stream_buffer_pos,len);
		  stream_buffer_pos=newpos;
		}
	}
}
