
#include "SDL.h"
#include "SDL_opengl.h"
#include <string.h>

#include "video3d.h"


/* tile pixels array */
extern Uint8 tiles[256][8][8];
extern Uint8 sprites[64][16][16];
extern Uint32 paldata[0x10];
extern int magictile;

typedef void (*draw_gl_obj)(int num, int color);

draw_gl_obj tileptr[256];
draw_gl_obj spriteptr[64];

/* GLU objects */
static GLUquadricObj *sphere;

static GLuint pill_list;
static GLuint powerpill_list;

/* each tile is extruded to to four display lists */
/* one for each colour */
static GLuint extrudedtiles; /* 256 tiles * 4 colors */
static GLuint extrudedsprites; /* 64 sprites * 4 colors */



void blank(int tile, int color) {
  return;
}


static void init_pill(void) {
    pill_list=glGenLists(1);
    glNewList(pill_list, GL_COMPILE);
    gluSphere(sphere,1.0,SLICES,STACKS);
    glEndList();
}

static void init_powerpill(void) {
    powerpill_list=glGenLists(1);
    glNewList(powerpill_list, GL_COMPILE);
    gluSphere(sphere,4.0,SLICES,STACKS);
    glEndList();
}

void pill(int tile, int color) {
  COLORGL(1);
  glCallList(pill_list);
}

void powerpill(int tile, int color) {
  COLORGL(1);
  glCallList(powerpill_list);
}

void draw_box(int fromX, int toX, int atY, int radius) {
    float x1,x2,y1,y2,z1,z2;

    x1 = (fromX - radius);
    x2 = (toX - radius);
    y1 = (atY - radius);
    y2 = (atY + 1 - radius);
    z1 = -0.5;
    z2 = 0.5;

    glBegin(GL_QUADS);
    /* top */
    glNormal3f(0.0,0.0,1.0);
    glVertex3f(x1,y1,z2);
    glVertex3f(x1,y2,z2);
    glVertex3f(x2,y2,z2);
    glVertex3f(x2,y1,z2);


    /* bottom */
    glNormal3f(0.0,0.0,-1.0);
    glVertex3f(x1,y1,z1);
    glVertex3f(x2,y1,z1);
    glVertex3f(x2,y2,z1);
    glVertex3f(x1,y2,z1);
    
    glEnd();
    
    /* sides */
    glBegin(GL_QUAD_STRIP);
    
    glVertex3f(x1,y1,z1);
    glVertex3f(x1,y1,z2);
    
    glVertex3f(x2,y1,z1);
    glVertex3f(x2,y1,z2);
    
    glVertex3f(x2,y2,z1);
    glVertex3f(x2,y2,z2);
    
    glVertex3f(x1,y2,z1);
    glVertex3f(x1,y2,z2);
    
    glVertex3f(x1,y1,z1);
    glVertex3f(x1,y1,z2);

    glEnd();

}


/* iterate over tiles and generate a display list for each color */
void calc_extruded_tiles (void) {
    int tile, x, y, color, from;

    extrudedtiles=glGenLists(256*4);

    for (tile=0; tile<256; tile++) {
        for (color=0;color<4; color++) {
            glNewList(extrudedtiles+(4*tile)+color, GL_COMPILE);
            for (y=0; y<8; y++) {
                from = -1;
                for (x=0; x<8; x++) {
                    /* test if this is the current color */
                    /* x values are flipped */
                    if (tiles[tile][y][7-x] == color) {
                        /* start marking box if not already */
                        if (from == -1) {
                            from = x;
                        }
                    } else if (from != -1) {
                        /* draw box */
                        draw_box(from,x,y,4);
                        from = -1;
                    }
                }
                if (from != -1) {
                    /* draw box to 8th pos */
                    draw_box(from,8,y,4);
                }
            }
            glEndList();
        }
    }
}

void draw_box2(int ix1, int ix2, int iy1, int iy2, int radius) {
    float x1,x2,y1,y2,z1,z2;
    x1 = ix1 - radius;
    x2 = ix2 - radius;
    y1 = iy1 - radius;
    y2 = iy2 - radius;
    z1 = -0.5;
    z2 = 0.5;

    glBegin(GL_QUADS);
    /* top */
    glNormal3f(0.0,0.0,1.0);
    glVertex3f(x1,y1,z2);
    glVertex3f(x1,y2,z2);
    glVertex3f(x2,y2,z2);
    glVertex3f(x2,y1,z2);


    /* bottom */
    glNormal3f(0.0,0.0,-1.0);
    glVertex3f(x1,y1,z1);
    glVertex3f(x2,y1,z1);
    glVertex3f(x2,y2,z1);
    glVertex3f(x1,y2,z1);
    
    glEnd();
    
    /* sides */
    glBegin(GL_QUAD_STRIP);
    
    glVertex3f(x1,y1,z1);
    glVertex3f(x1,y1,z2);
    
    glVertex3f(x2,y1,z1);
    glVertex3f(x2,y1,z2);
    
    glVertex3f(x2,y2,z1);
    glVertex3f(x2,y2,z2);
    
    glVertex3f(x1,y2,z1);
    glVertex3f(x1,y2,z2);
    
    glVertex3f(x1,y1,z1);
    glVertex3f(x1,y1,z2);

    glEnd();

}



/* iterate over tiles and generate a display list for each color */
void calc_extruded_tiles2 (void) {
    int tile, x1, y1, x2, y2, color;
    Uint8 t[8][8];

    extrudedtiles=glGenLists(256*4);

    for (tile=0; tile<256; tile++) {
        for (color=0;color<4; color++) {
            memcpy(t,tiles[tile],sizeof(t));
            glNewList(extrudedtiles+(4*tile)+color, GL_COMPILE);
            for (y1=0; y1<8; y1++) {
                x1 = -1;
                for (x2=0; x2<9; x2++) {
                    /* test if this is the current color */
                    /* x values are flipped */
                    if (x2<8 && (t[y1][7-x2] == color)) {
                        /* start marking box if not already */
                        if (x1 == -1) {
                            x1 = x2;
                        }
                    } else if (x1 != -1) {
                        /* test for box y extension */
                        for (y2=y1+1; y2<9; y2++) {
                           if (y2<8 && !memcmp(&t[y1][8-x2], &t[y2][8-x2], x2-x1)) {
                              memset(&t[y2][8-x2],255,x2-x1);
                           } else {
                              break;
                           }
                        }
                        draw_box2(x1,x2,y1,y2,4);
                        x1 = -1;
                    }
                }
            }
            glEndList();
        }
    }
}

void calc_extruded_sprites (void) {
    int sprite, x, y, color, from;

    extrudedsprites=glGenLists(64*4);
    for (sprite=0; sprite<64; sprite++) {
        for (color=0;color<4; color++) {
            glNewList(extrudedsprites+(sprite*4)+color, GL_COMPILE);
            for (y=0; y<16; y++) {
                from = -1;
                for (x=0; x<16; x++) {
                    /* test if this is the current color */
                    /* x values are flipped */
                    if (sprites[sprite][y][15-x] == color) {
                        /* start marking box if not already */
                        if (from == -1) {
                            from = x;
                        }
                    } else if (from != -1) {
                        /* draw box */
                        draw_box(from,x,y,8);
                        from = -1;
                    }
                }
                if (from != -1) {
                    /* draw box to 16th pos */
                    draw_box(from,16,y,8);
                }
            }
            glEndList();
        }
    }
}

void calc_extruded_sprites2 (void) {
    int sprite, x1, y1, x2, y2, color;
    Uint8 s[16][16];

    extrudedsprites=glGenLists(64*4);

    for (sprite=0; sprite<64; sprite++) {
        for (color=0;color<4; color++) {
            memcpy(s,sprites[sprite],sizeof(s));
            glNewList(extrudedsprites+(4*sprite)+color, GL_COMPILE);
            for (y1=0; y1<16; y1++) {
                x1 = -1;
                for (x2=0; x2<17; x2++) {
                    /* test if this is the current color */
                    /* x values are flipped */
                    if (x2<16 && (s[y1][15-x2] == color)) {
                        /* start marking box if not already */
                        if (x1 == -1) {
                            x1 = x2;
                        }
                    } else if (x1 != -1) {
                        /* test for box y extension */
                        for (y2=y1+1; y2<17; y2++) {
                           if (y2<16 && !memcmp(&s[y1][16-x2], &s[y2][16-x2], x2-x1)) {
                              memset(&s[y2][16-x2],255,x2-x1);
                           } else {
                              break;
                           }
                        }
                        draw_box2(x1,x2,y1,y2,8);
                        x1 = -1;
                    }
                }
            }
            glEndList();
        }
    }
}



/* draw the extruded tile */
/* black is transparent */
void draw_extruded_tile (int tile, int color) {
    GLuint tilelists=extrudedtiles+(4*tile);
    if (paldata2[color+0] != 0) {
        COLORGL(0);
        glCallList(tilelists+0);
    }
    if (paldata2[color+1] != 0) {
        COLORGL(1);
        glCallList(tilelists+1);
    }
    if (paldata2[color+2] != 0) {
        COLORGL(2);
        glCallList(tilelists+2);
    }
    if (paldata2[color+3] != 0) {
        COLORGL(3);
        glCallList(tilelists+3);
    }
}

void draw_extruded_sprite (int sprite, int color) {
    int spritelists=extrudedsprites+(4*sprite);
    if (colortable[color+0] != 0) {
        COLORGL(0);
        glCallList(spritelists+0);
    }
    if (colortable[color+1] != 0) {
        COLORGL(1);
        glCallList(spritelists+1);
    }
    if (colortable[color+2] != 0) {
        COLORGL(2);
        glCallList(spritelists+2);
    }
    if (colortable[color+3] != 0) {
        COLORGL(3);
        glCallList(spritelists+3);
    }
}



void draw_3d_tile (int tile, int color) {
  /* check for special model */
  /*if (tile == magictile) {
      color = (color + 1)%0x76;
  }
  if (tileptr[tile] != NULL) {*/
    (tileptr[tile])(tile, color);
/*  } else {
    draw_extruded_tile(tile, color);
  }*/
}

void draw_3d_sprite(int sprite, int color) {
 /* if (spriteptr[sprite]!=NULL) {*/
    (spriteptr[sprite])(sprite, color);
/*  } else {
    draw_extruded_sprite(sprite, color);
  }*/
}


void init_pacman_3d_objs(void) {
 int i ;

  sphere= gluNewQuadric();

  for(i=0; i<256; i++) {
    tileptr[i]=draw_extruded_tile;
  }
  for(i=0; i<64; i++) {
    spriteptr[i]=draw_extruded_sprite;
  }

  tileptr[16]=pill;
  tileptr[20]=powerpill;

  spriteptr[28]=draw_scared_ghost;/* scared ghost 1 */
  spriteptr[29]=draw_scared_ghost;/* scared ghost 2 */

  spriteptr[30]=blank;
  spriteptr[31]=blank;

  spriteptr[32]=draw_ghost;       /* eyes right 1 */
  spriteptr[33]=draw_ghost;       /* eyes right 2 */
  spriteptr[34]=draw_ghost;       /* eyes down  1 */
  spriteptr[35]=draw_ghost;       /* eyes down  2 */
  spriteptr[36]=draw_ghost;       /* eyes left  1 */
  spriteptr[37]=draw_ghost;       /* eyes left  2 */
  spriteptr[38]=draw_ghost;       /* eyes up    1 */
  spriteptr[39]=draw_ghost;       /* eyes up    2 */
  /* 40 = 200 */
  /* 41 = 400 */
  /* 42 = 800 */
  /* 43 = 1600 */
  spriteptr[44]=draw_pacman;      /* left wide */
  spriteptr[45]=draw_pacman;      /* down wide */
  spriteptr[46]=draw_pacman;      /* left narrow */
  spriteptr[47]=draw_pacman;      /* down narrow */
  spriteptr[48]=draw_pacman;      /* whole */
  
  spriteptr[52]=draw_pacman;      /* die 1 */
  spriteptr[53]=draw_pacman;      /* die 2 */
  spriteptr[54]=draw_pacman;      /* die 3 */
  spriteptr[55]=draw_pacman;      /* die 4 */
  spriteptr[56]=draw_pacman;      /* die 5 */
  spriteptr[57]=draw_pacman;      /* die 6 */
  spriteptr[58]=draw_pacman;      /* die 7 */
  spriteptr[59]=draw_pacman;      /* die 8 */
  spriteptr[60]=draw_pacman;      /* die 9 */
  spriteptr[61]=draw_pacman;      /* die 10 */
  spriteptr[62]=draw_pacman;      /* die 11 */
  
  spriteptr[63]=blank;            /* blank */

  calc_extruded_tiles2();
  calc_extruded_sprites2();
  construct_pacman();
  construct_ghost();
  init_pill();
  init_powerpill();
   
}




